#  swiftgraph

A command-line tool to generate dependency graphs from annotated source files.

Usage:
```
swiftgraph <source folder>
```

The tool will recursively scan the folder, collecting information about types carrying special annotations, 
and write a `components.dot` file  that can then be run through graphviz to create SVG or PNG output.

Here's how to read the rendered output:

Bubbles:

- the yellow bubbles are the components
- the white rectangles are the configurations

Lines:

- solid lines are strong references
- dashed lines are weak references
- dotted lines are references (at least, conceptually, because configurations can be value types) to configurations
- red lines are dependency rule violations (see below)

The tool will log any warnings to standard out. These can then be piped into other tools to, for examle, 
trigger Slack notifications.

## Annotations

Currently, the following annotations are defined:

- `#@component`: a _component_ contains business logic. Components are classes (reference types).
- `#@configuration`:  a _configuration_ contains identifiers, network environment settings, and the like. Configurations can be structs (value types) or classes (reference types).

Note: the `#@` prefix has been chosen because it is not used anywhere else in our codebase, so it's easy to do a 
wildcard search for annotations on a large number of files.

The  `#@component` and  `#@configuration` annotations can be placed anywhere in the documentation header
of a class or struct. Properties of components in turn referencing other components or configurations do not need 
to be annotated – all (private or public) properties of these types are considered.


## Dependency Rules

- `#@component` marks a component class
  * a component must be class
  * a component can reference any other component
  * a component can reference any configuration
- `#@configuration` marks a configuration class or struct
  * a configuration can be a value type
  * a configuration may embed or reference other configurations
  * a configuration must not reference any components
  
Hidden dependencies (e.g., use of singletons in methods) must be made explicit via properties, e.g.
  
```
/// make hidden dependencies explicit
private weak var _sessionModule: RTSessionModule?
private weak var _gpsModule: RTGPSModule?
```

Note here that a dependency chain `A -> B.C.D` is really a dependency `A -> D` (because `D` could be injected into `A`), so
you want to point out the _last_ type in this chain for hidden dependencies.


## Backlog

#### Component and Configuration Attributes

Add support for `label=value` pairs on the annotations, e.g.

```
/// #@component[pod=network]
extension RTNetworkEngine { }
```

#### Group components into clusters

Building on the attributes, support grouping components into [subgraphs](https://graphviz.gitlab.io/_pages/Gallery/directed/cluster.html) (e.g., by cluster) in the dot output.

```
/// #@component[cluster=user]
final class SyncManager: NSObject {
...

/// #@component[cluster=user]
final class LoginCoordinator: NSObject, LoginPushPermissionDelegate {
...
```

## Misc

Online graphviz renderers:
- https://dreampuf.github.io/GraphvizOnline

