//
//  main.swift
//  swiftgraph
//
//  Created by Karl Traunmüller on 14.10.19.
//  Copyright © 2019 Runtastic. All rights reserved.
//

import Foundation

import SwiftGraphFramework

let arguments = Array(CommandLine.arguments.dropFirst())
let skipWeakReferences = isFlagSet(.skipWeakReferences, in: arguments)
let skipConfigurationReferences = isFlagSet(.skipConfigurationReferences, in: arguments)

guard let path = arguments.last else {
    printHelp()
    exit(1)
}

let scanner = AnnotationScanner()
guard let dependencyInfo = try scanner.scanFolder(at: path) else {
    exit(1)
}

let edges = makeDependencyGraph(dependencyInfo: dependencyInfo)

let dot = makeDot(dependencyInfo: dependencyInfo,
                  edges: edges,
                  skipWeakReferences: skipWeakReferences,
                  skipConfigurationReferences: skipConfigurationReferences)

let dotFileUrl = URL(fileURLWithPath: (path as NSString).appendingPathComponent("components.dot"))
print("writing dotfile at \(dotFileUrl.absoluteString)")
try dot.write(to: dotFileUrl, atomically: true, encoding: .utf8)

//print(dot)

for warning in dependencyInfo.warnings {
    print(warning)
}

exit(0)
