//
//  DotWriter.swift
//  swiftgraph
//
//  Created by Karl Traunmüller on 15.10.19.
//  Copyright © 2019 Runtastic. All rights reserved.
//

public func makeDot(dependencyInfo: DependencyInfo,
                    edges: [DependencyGraphEdge],
                    skipWeakReferences: Bool = false,
                    skipConfigurationReferences: Bool = false) -> String {
    let indent = "  "
    var dot = "digraph {\n"
    dot.append(indent + "// configurations\n")
    for configuration in dependencyInfo.configurations.sorted(by: { lhs, rhs in lhs.typeName < rhs.typeName }) {
        dot.append(indent + "\(configuration.typeName)[shape=box]\n")
    }
    dot.append(indent + "// components\n")
    for component in dependencyInfo.components.sorted(by: { lhs, rhs in lhs.typeName < rhs.typeName }) {
        dot.append(indent + "\(component.typeName)[fillcolor=lightyellow,style=filled]\n")
    }
    dot.append(indent + "// edges\n")
    for edge in edges {
        if (skipWeakReferences && edge.kind == .weakComponentReference) ||
            (skipConfigurationReferences && edge.kind == .configurationReference) {
            continue
        }
        dot.append(indent + "\(edge.fromTypeName) -> \(edge.toTypeName)")
        switch edge.kind {
        case .strongComponentReference:
            break
        case .weakComponentReference:
            dot.append("[style=dashed]")
        case .configurationReference:
            dot.append("[style=dotted]")
        case .ruleViolation:
            dot.append("[color=red,style=bold]")
        }
        dot.append("\n")
    }
    dot.append("}")
    return dot
}
