//
//  GraphBuilder.swift
//  swiftgraph
//
//  Created by Karl Traunmüller on 15.10.19.
//  Copyright © 2019 Runtastic. All rights reserved.
//

/// Note: the return type could be a Set, but we want a stable order of edges in the graph
/// (not just for repeatable tests, also for diff'ing) - that's why Array is used.
public func makeDependencyGraph(dependencyInfo: DependencyInfo) -> [DependencyGraphEdge] {
    var edges = Set<DependencyGraphEdge>()
    for fromComponent in dependencyInfo.components {
        for fromProperty in fromComponent.properties {
            for toComponent in dependencyInfo.components {
                if fromProperty.typeName == toComponent.typeName {
                    edges.insert(DependencyGraphEdge(fromTypeName: fromComponent.typeName,
                                                     toTypeName: toComponent.typeName,
                                                     kind: fromProperty.isWeak ? .weakComponentReference : .strongComponentReference))
                }
            }
            for toConfiguration in dependencyInfo.configurations {
                if fromProperty.typeName == toConfiguration.typeName {
                    edges.insert(DependencyGraphEdge(fromTypeName: fromComponent.typeName,
                                                     toTypeName: toConfiguration.typeName,
                                                     kind: .configurationReference))
                }
            }
        }
    }
    for fromConfiguration in dependencyInfo.configurations {
        for fromProperty in fromConfiguration.properties {
            for toComponent in dependencyInfo.components {
                if fromProperty.typeName == toComponent.typeName {
                    edges.insert(DependencyGraphEdge(fromTypeName: fromConfiguration.typeName,
                                                     toTypeName: toComponent.typeName,
                                                     kind: .ruleViolation))
                    print("Dependency violation: configuration \(fromConfiguration.typeName) references component \(toComponent.typeName) in property \(fromProperty.name)")
                }
            }
            for toConfiguration in dependencyInfo.configurations {
                if fromProperty.typeName == toConfiguration.typeName {
                    edges.insert(DependencyGraphEdge(fromTypeName: fromConfiguration.typeName,
                                                     toTypeName: toConfiguration.typeName,
                                                     kind: .configurationReference))
                }
            }
        }
    }
    return edges.sorted(by: { lhs, rhs in
        return lhs.fromTypeName + lhs.toTypeName < rhs.fromTypeName + rhs.toTypeName
    })
}
