//
//  UnitTests.swift
//  UnitTests
//
//  Created by Karl Traunmüller on 14.10.19.
//  Copyright © 2019 Runtastic. All rights reserved.
//

import XCTest

@testable import SwiftGraphFramework

final class AnnotationScannerTests: XCTestCase {
    
    func testScanComponent() throws {
        guard let (_, contents) = loadSample(named: "AppManager") else {
            return XCTFail("could not load sample file")
        }
        let scanner = AnnotationScanner()
        guard let dependencyInfo = try scanner.scanFile(fileContents: contents) else {
            return XCTFail()
        }
        
        guard let appManager = dependencyInfo.firstComponentOfType("AppManager") else {
            return XCTFail("expected component 'AppManager'")
        }
        
        // Rose
        guard let rose = appManager.firstPropertyOfType("Rose") else {
            return XCTFail()
        }
        XCTAssertEqual(rose.name, "rose")
        XCTAssertFalse(rose.isStatic)
        XCTAssertFalse(rose.isOptional)
        XCTAssertFalse(rose.isWeak)
        
        // Environment
        guard let environment = appManager.firstPropertyOfType("Environment") else {
            return XCTFail()
        }
        XCTAssert(environment.isStatic)
        
        // GoalsModule
        guard let goalsModule = appManager.firstPropertyOfType("GoalsModule") else {
            return XCTFail()
        }
        XCTAssert(goalsModule.isOptional)
        
        // SyncManager
        guard let syncManager = appManager.firstPropertyOfType("SyncManager") else {
            return XCTFail()
        }
        XCTAssert(syncManager.isWeak)
        
        // ApplicationSettings
        guard appManager.firstPropertyOfType("ApplicationSettings") != nil else {
            return XCTFail()
        }
        
        // VoiceCoachController
        guard appManager.firstPropertyOfType("VoiceCoachController") == nil else {
            return XCTFail()
        }
        XCTAssertFalse(appManager.warnings.isEmpty)
    }
}
