//
//  AppManager.swift
//  runtastic
//
//  Created by Daniel Holzmann on 16/03/2017.
//  Copyright © 2017 runtastic GmbH. All rights reserved.
//

import Foundation

import RTCommon
import RTFeatureFlags
import RTFrameworkEntitySync
import RTFrameworkInternetReachability
import RTFrameworkSession
import RTFrameworkNetwork
import RTFrameworkTracking
import RTFrameworkUser
import RTGoals
import RTRemoteConfig
import RTRose

/// This class holds the important objects used for:
///
/// - Manage and change environment dynamically
/// - Application constants
/// - Framework manager (RTFrameworkManager)
///
/// #@component
@objc(RTAppManager)
@objcMembers
final class AppManager: NSObject {
    
    let rose: Rose
    
    @objc
    var currentEnvironmentName: String {
        return rose.currentEnvironment.name
    }
    
    let constants: ApplicationSettings
    let frameworkManager: RTFrameworkManager
    
    let remoteConfigValues: RemoteConfigValues

    let voiceController = VoiceCoachController()
    
    private(set) var configuration: RegionalAppConfiguration
    
    private(set) weak var syncManager: SyncManager?
    private(set) var trainingPlansInteractor: TrainingPlansInteractor?
    
    var goalsModule: GoalsModule?
    
    override init() {
    }

    func startFramework() {
    }
}

/// #@component
extension AppManager {
    static var developmentEnvironment: Environment {
        return RTHelper.isRunningEndToEndTests() ? .preProduction : .production
    }
}

/// Declare network's Rose a configuration type:
/// #@configuration[pod=network]
extension Rose {
}

/// Declare network's Environment a configuration type:
/// #@configuration[pod=network]
extension Environment {
}

/// Declare this Objective-C type a component type:
/// #@component
extension RTFrameworkManager {
}

/// Declare goals's GoalsModule a component type:
/// #@component[pod=goals]
extension GoalsModule {
}

/// #@component
extension SyncManager {
}
