//
//  AppSettings.swift
//  runtastic
//
//  Created by Alex Manzella on 11/05/16.
//  Copyright © 2016 runtastic GmbH. All rights reserved.
//

import Foundation

import Adjust

import RTCommon
import RTFrameworkNetwork
import RTSettings

/// #@configuration
@objcMembers
final class ApplicationSettings: NSObject {
    
    var appManager: AppManager // this is here just to check for dependency rule violations
    
    let environment: Environment
    let serviceRegion: ServiceRegion
    let appName = "runtastic"
    
    static let firstLaunchDateDefaultsKey = "FirstLaunchDate"
    
    // MARK: Runtastic
    
    let serverAppId = "2637" // Runtastic Lite 10.3
    let serverAppUid = "O04a50e303d1572203dc252dfb9e85a01ee9de150c2e7baf0702f80275c580b2" // Runtastic Lite 10.3
    
    var webPortalBaseURL: String {
        return environment.urls.web
    }
    
    // MARK: FB
    
    var facebookAppId: String {
        return "162918433202"
    }
    
    var facebookTrackingAppId: String {
        return "535415513277333"
    }
    
    // MARK: Google
    
    var googleAnalyticsTrackingId: String {
        #if DEBUG
            return "UA-42210135-2"
        #else
            return "UA-41119181-1"
        #endif
    }
    
    var googleClientId: String {
        return "190569399906-n7q98tmbt19k52q64aqamcsg7t8at8ra.apps.googleusercontent.com"
    }
    
    var googleClientServerID: String {
        return "190569399906-vq8qg9l1mvk4u5ahju38g5aiodvepmv4.apps.googleusercontent.com"
    }
    
    // MARK: Twitter

    let twitterConsumerKey = "vmZmO7uDjgNT5fODpZeRHpPsV"
    let twitterConsumerSecret = "0O0AnboCT4rJbb1t4TYs9Dg6duorv925xHbE3P9661ets7qQNc"
    
    // MARK: WeChat
    
    var weChatAppId: String {
        return "wx1e417fe837654f42"
    }
    
    // MARK: Pushwoosh
    
    var pushwooshAppCode: String {
        #if DEBUG
            return "84640-27909"
        #else
            return "298A4-2F32B"
        #endif
    }
    
    var pushwooshAppName: String {
        #if DEBUG
            return "Runtastic Staging"
        #else
            return "Runtastic"
        #endif
    }
    
    private var isProductionEnvironment: Bool {
        if case .production = environment {
            #if DEBUG
                // never report to production systems in DEBUG
                return false
            #else
                return true
            #endif
        }
        
        return false
    }
    
    // MARK: Adjust
    
    var adjustEnvironment: String {
        let environment = RTSettings.shared.adjustEnvironment ?? "sandbox"
        return environment
    }
    
    // MARK: CRM
    
    var isCRMDebugEnvironment: Bool {
        return RTSettings.shared.crmEnvironment == "testing"
    }

    // MARK: Apple Music
    
    /// Will expire in 26.09.2019 + 6 months ;)
    let appleMusicDeveloperToken = "eyJhbGciOiJFUzI1NiIsInR5cCI6IkpXVCIsImtpZCI6IjdXWkE3UTdGS0cifQ.eyJpYXQiOjE1Njk1MDU1MzIsImV4cCI6MTU4NTIzMDMzMiwiaXNzIjoiVFBMRjIzNFdZWSJ9.xZ2pmNSwhH5XKPBk6Wrfg7pPT_hiHs0074hSgV7FDuZp7IUu0rBoaDe_YPQPE4oVEThhJF5HaIQVS6-1WUlqwg"
    
    let appleMusicPlaylistIds = ["pl.67659f652eaa460e9eda905b79ad91c9",
                                 "pl.d43d1ba2aa9d4df8aaa598f103254918",
                                 "pl.14457d9e86fc421f9580c570f4d2ea35",
                                 "pl.204a32bda95341b2958e981acef89533"]
    
    let appleMusicHighlightedPlaylistId = ["pl.204a32bda95341b2958e981acef89533"]
    let appleMusicAffiliateToken = "11l79t"
    
    // Redeem code affiliate program (RUN-13452)
    let appleMusicClientId = "5b7348464810136526cfc3b0"
    let appleMusicSecretKey = "3FeR1pv+psCn6V28685qZ/fT6ROR0TM86KshJXBdiUurzPjuWpywiAYlqJrdhgZmzqrEgpdi46xYXq6Td9EwzQ=="
    
    init(environment: Environment, serviceRegion: ServiceRegion) {
        self.environment = Environment(environment, serviceRegion: serviceRegion)
        self.serviceRegion = serviceRegion
    }
}

/// Declare common's RTServiceRegion a configuration type:
/// #@configuration[pod=common]
extension ServiceRegion {
}
